--########################################################################
--
--                               CIRCUITO TOP
--
--   FERNANDO GEHM MORAES	03/agosto/2005
--   NEY LAERT VILAR CALAZANS	16/junho/2006 - Adaptao para D2SB/DIO4
--				e acrscimo de entradas para controles
--

--########################################################################
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_Logic_unsigned.all;   -- uso conv_integer
use IEEE.Std_Logic_arith.all;
 
entity top is
          port( ck :      in  std_logic;
                reset:    in  std_logic; -- Ney: active-high reset
                dipsw:    in  std_logic_vector(7 downto 0);
                btns:	  in  std_logic_vector(5 downto 1); 
                -- added by Ney - for DIO4, five push-buttons BTN5 to BTN1
                LEDG :    out std_logic; 
                -- added by Ney - for DIO4, active-high LED enable needed
		halt :    out std_logic;
		-- added by Ney - for D2SB, there is a led left, use it!
                leds:     out std_logic_vector(7 downto 0);
                an :      out std_logic_vector(4 downto 1);
                dec_ddp : out std_logic_vector(7 downto 0)
             );
end top;
        

architecture top of top is
                                      
   signal hold, ce, rw, ceCPU, rwCPU, ceES, rwES: std_logic;
   signal address, 
   	  datainCPU,
   	  dataoutCPU,
   	  dataoutRAM,
   	  d1_CPU, -- output to two displays
   	  d2_CPU  -- output to two displays
   	  : std_logic_vector(7 downto 0);
  
   signal d1,d2,d3,d4: std_logic_vector(6 downto 0);

   signal  CLK_DIV : std_logic;

begin

   -- The 50MHz clock of the board is too fast for the Clepatra processor 
   -- to run without careful logic/physical synthesis. To avoid problems
   -- the process below simply divides the clock by 2 (from 50MHz to 25MHz)
   -- A better option would be to optimize the synthesis to make it run at 
   -- 50MHz, or at least to use a DLL to divide the clock properly.
   -- For teaching purposes this process is good enough
   --
   -- Added by Ney (18/06/2006)
   --
   process (ck)
   begin
	if ck'event and ck='1' then
		CLK_DIV <= not CLK_DIV;
	end if;	
   end process;

   -- Line below was added by Ney to enable the latch controlling the 
   -- lighting of the leds in board DIO4
   LEDG <= '1'; 

   -------------------------------------------------------------------------------
   -- Aqui, insere-se a instncia o processador Clepatra
   --
   cpuC : entity work.cleopatra port map
               ( clock=>CLK_DIV, reset=>reset, halt=>halt, hold=>hold,
                 ce=>ce, rw=>rw, address=>address ,datain=>datainCPU, dataout=>dataoutCPU);
   -- Para garantir que processador no  impedido de progredir, coloca-se hold em 0
   hold <= '0';
	
   -------------------------------------------------------------------------------
   -- Aqui, insere-se a instncia da memria, j com o programa inicializado
   --
   ramC: entity work.RAM_mem port map
               ( clock=>CLK_DIV, ce=>ceCPU,rw=>rwCPU, address=>address,
               datain=>dataoutCPU, dataout=>dataoutRAM);
 
 
   -------------------------------------------------------------------------------
   -- instancia o decodificador que define se  acesso  memria ou a entrada/dada
   --
   deco: entity work.es_decoder port map
             ( ce => ce, rw=>rw, address => address,
               ceCPU=>ceCPU, rwCPU=>rwCPU, ceES=>ceES, rwES=>rwES);
 
   -------------------------------------------------------------------------------
   -- Comando que captura o dado de algum dispositivo externo  CPU Clepatra.
   -- Este comando  disparado por uma instruo LDA executada pela Clepatra.
   --
   -- O dado lido pode vir da memria ou de algum disppositivo de entrada, quais
   -- sejam as 8 chaves deslizantes ou uma combinao dos 5 botes BTN1 a BTN5
   -- da DIO4. Recomenda-se usar as chaves deslizantes para entrada de dados e 
   -- os botes para entrada de sinais de controle.
   -- Mapa de leitura:
   --	Leitura de endereos 00h a 0FBh => leitura de posio da memria principal
   --	Leitura de FC => captura valor de BTN5 a BTN1 da DIO4 colocando nos bits 
   --			 respectivos do Acumulador (5 downto 1)
   --	Leitura de FD => captura valor das chaves deslizantes
   --	Leitura de FE => sem efeito nesta implementao
   --	Leitura de FF => sem efeito nesta implementao
   --
   datainCPU <= dataoutRAM 		when ceCPU='1' else 
                "00" & btns & '0'  	when ceES='1' and address=x"FC" else
                -- Coloca no acumulador o valor dos botes BTN5 a BTN1
                dipsw			when ceES='1' and address=x"FD" else
                -- Coloca no acumulador o valor das chaves
                (others=>'Z');
    

   -------------------------------------------------------------------------------
   -- Processo que captura o dado escrito pela CPU em dispositivos de sada 
   -- 
   -- d1_CPU, d2_CPU e leds so trs registradores de 8 bits que recebem
   --	dados via instrues STA do processador e disponibilizam sua sadas em 
   --	dispositivos de E/S (displays e leds).
   -- Mapa de escrita:
   --	Escrita em endereos 00h a 0FBh => escrita em posio da memria principal
   --	Escrita em FC => sem efeito nesta implementao
   --	Escrita em FD => efeito nos 8 leds da DIO4
   --	Escrita em FE => efeito nos 2 displays mais  esquerda da DIO4
   --	Escrita em FF => efeito nos 2 displays mais  direita da DIO4
   --
   process (CLK_DIV, reset)
    begin
         if reset='1' then
              d1_CPU  <= (others=>'0');
              d2_CPU  <= (others=>'0');
              leds    <= (others=>'0');            
         elsif CLK_DIV'event and CLK_DIV='1' then
              if ceES='1' and  rwES='0' then 
                   -- seno, envia para os leds o contedo do acumulador
                   if address=x"FD" then leds   <= dataoutCPU; end if; 
                   if address=x"FE" then d1_CPU <= dataoutCPU; end if;
                   if address=x"FF" then d2_CPU <= dataoutCPU; end if;  
              end if;  
         end if;
   end process;
 
   -------------------------------------------------------------------------------
   -- Display Interface Instantiation   D1  D2   D3  D4
   --
   d1 <= "10" & d1_CPU(7 downto 4) & '1'; -- Ney: leftmost display in DIO4
   d2 <= "10" & d1_CPU(3 downto 0) & '1'; -- Ney: display left to center in DIO4
   d3 <= "10" & d2_CPU(7 downto 4) & '1'; -- Ney: display right to center in DIO4
   d4 <= "10" & d2_CPU(3 downto 0) & '1'; -- Ney: rightmost display in DIO4
     
   DISPLAY : entity work.dio4_dspl_drv port map
              ( clock => ck,  reset => reset, d4 => d4, d3 => d3, d2 => d2,  d1 => d1,
                an => an,  dec_ddp => dec_ddp );  

end top;
